package com.sun.java.swing.plaf.motif;

import java.awt.Point;
import java.awt.Container;
import javax.swing.JMenuBar;
import javax.swing.MenuSelectionManager;
import java.awt.event.MouseEvent;
import javax.swing.JMenuItem;
import javax.swing.event.ChangeEvent;
import javax.swing.event.MouseInputListener;
import javax.swing.JPopupMenu;
import javax.swing.MenuElement;
import javax.swing.JMenu;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.plaf.basic.BasicMenuUI;

public class MotifMenuUI extends BasicMenuUI
{
    public MotifMenuUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MotifMenuUI();
    }
    
    @Override
    protected ChangeListener createChangeListener(JComponent component) {
        return new MotifMenuUI.MotifChangeHandler((JMenu)component, this);
    }
    
    private boolean popupIsOpen(JMenu menu, MenuElement[] array) {
        JPopupMenu popupMenu = menu.getPopupMenu();
        int i = array.length - 1;
        while (i >= 0) {
            if (array[i].getComponent() == popupMenu) {
                return true;
            }
            --i;
        }
        return false;
    }
    
    @Override
    protected MouseInputListener createMouseInputListener(JComponent component) {
        return new MotifMenuUI.MouseInputHandler();
    }
    
    public class MotifChangeHandler extends BasicMenuUI.ChangeHandler
    {
        public MotifChangeHandler(JMenu m, MotifMenuUI ui) {
            super(m, ui);
        }
        
        @Override
        public void stateChanged(ChangeEvent e) {
            JMenuItem menuItem = (JMenuItem)e.getSource();
            if (menuItem.isArmed() || menuItem.isSelected()) {
                menuItem.setBorderPainted(true);
            }
            else {
                menuItem.setBorderPainted(false);
            }
            super.stateChanged(e);
        }
    }
    
    protected class MouseInputHandler implements MouseInputListener
    {
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            JMenu menu = (JMenu)mouseEvent.getComponent();
            if (menu.isEnabled()) {
                if (menu.isTopLevelMenu()) {
                    if (menu.isSelected()) {
                        defaultManager.clearSelectedPath();
                    }
                    else {
                        Container parent = menu.getParent();
                        if (parent != null && parent instanceof JMenuBar) {
                            defaultManager.setSelectedPath(new MenuElement[] { (MenuElement)parent, menu });
                        }
                    }
                }
                MenuElement[] path = MotifMenuUI.this.getPath();
                if (path.length > 0) {
                    MenuElement[] selectedPath = new MenuElement[path.length + 1];
                    System.arraycopy(path, 0, selectedPath, 0, path.length);
                    selectedPath[path.length] = menu.getPopupMenu();
                    defaultManager.setSelectedPath(selectedPath);
                }
            }
        }
        
        @Override
        public void mouseReleased(MouseEvent event) {
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            JMenuItem menuItem = (JMenuItem)event.getComponent();
            Point point = event.getPoint();
            if (point.x < 0 || point.x >= menuItem.getWidth() || point.y < 0 || point.y >= menuItem.getHeight()) {
                defaultManager.processMouseEvent(event);
            }
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseDragged(MouseEvent event) {
            MenuSelectionManager.defaultManager().processMouseEvent(event);
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
        }
    }
}
